@@ -28,7 +28,7 @@
 //! ## Code you write ...
 //!
 //! ```rust
-//! use interoptopus::{ffi_function, ffi_type, inventory};
+//! use interoptopus::{ffi_function, ffi_type, Inventory, InventoryBuilder, function};
 //!
 //! #[ffi_type]
 //! #[repr(C)]
@@ -43,24 +43,28 @@
 //!     println!("{}", input.x);
 //! }
 //!
-//! // This defines our FFI interface as `ffi_inventory` containing
-//! // no constants, a single function `my_function`, no additional
-//! // types (types are usually inferred) and no codegen patterns.
-//! inventory!(ffi_inventory, [], [my_function], [], []);
+//! // Define our FFI interface as `ffi_inventory` containing
+//! // a single function `my_function`. Types are inferred.
+//! pub fn ffi_inventory() -> Inventory {
+//!     InventoryBuilder::new()
+//!         .register(function!(my_function))
+//!         .inventory()
+//! }
 //!
 //! ```
 //!
 //!
 //! ## ... Interoptopus generates
 //!
-//! | Language | Crate | Sample Output |
+//! | Language | Crate | Sample Output<sup>1</sup> |
 //! | --- | --- | --- |
 //! | C# | [**interoptopus_backend_csharp**](https://crates.io/crates/interoptopus_backend_csharp) | [Interop.cs](https://github.com/ralfbiedert/interoptopus/blob/master/backends/csharp/tests/output_safe/Interop.cs) |
 //! | C | [**interoptopus_backend_c**](https://crates.io/crates/interoptopus_backend_c) | [my_header.h](https://github.com/ralfbiedert/interoptopus/blob/master/backends/c/tests/output/my_header.h) |
 //! | Python | [**interoptopus_backend_cpython**](https://crates.io/crates/interoptopus_backend_cpython) | [reference.py](https://github.com/ralfbiedert/interoptopus/blob/master/backends/cpython/tests/output/reference_project.py) |
-//! | Other | Write your own backend<sup>1</sup> | - |
+//! | Other | Write your own backend<sup>2</sup> | - |
 //!
-//! <sup>1</sup> Create your own backend in just a few hours. No pull request needed. [Pinkie promise](https://github.com/ralfbiedert/interoptopus/blob/master/FAQ.md#new-backends).
+//! <sup>1</sup> For the [reference project](https://github.com/ralfbiedert/interoptopus/tree/master/reference_project/src). <br/>
+//! <sup>2</sup> Create your own backend in just a few hours. No pull request needed. [Pinkie promise](https://github.com/ralfbiedert/interoptopus/blob/master/FAQ.md#new-backends).
 //!
 //!
 //!
@@ -121,6 +125,7 @@
 //!
 //! ## Changelog
 //!
+//! - **v0.14** - Better inventory UX.
 //! - **v0.13** - Python backend uses `ctypes` now.
 //! - **v0.12** - Better compat using `#[ffi_service_method]`.
 //! - **v0.11** - C# switch ctors to static methods.
@@ -162,7 +167,7 @@
 //! [docs]: https://docs.rs/interoptopus/badge.svg
 //! [docs.rs]: https://docs.rs/interoptopus/
 
-pub use crate::core::{merge_libraries, Library};
+pub use crate::core::{merge_inventories, non_service_functions, Inventory, InventoryBuilder, Symbol};
 pub use error::Error;
 pub use generators::Interop;
 #[cfg(feature = "derive")]
@@ -192,127 +197,169 @@ pub mod lang {
     pub mod rust;
 }
 
-/// **The** macro to define your library, ties everything together!
+/// Register a function with an [`InventoryBuilder`].
 ///
-/// This macro produces an "inventory function", which can be ingested by backends. The function
-/// will have the signature `fn f() -> Library`, where [`Library`] represents all functions,
-/// types, constants and documentation exported by this crate over the FFI boundary.
-///
-/// # Usage
-///
-/// This macro must be invoked with exactly 5 parameters:
-///
-/// ```ignore
-/// # use interoptopus::inventory_function;
-/// inventory_function!(symbol, consts, functions, extra_types, patterns);
-/// ```
-///
-/// Where
-/// - `symbol` - the name of the exported inventory function producing a [`Library`],
-/// - `consts` - a list of [`#[ffi_constant]`](crate::ffi_constant) constants to include `[C1, C2, ...]`,
-/// - `functions` - a list of [`#[ffi_function]`](crate::ffi_function) functions to include `[f1, f2, ...]`,
-/// - `extra_types` - additional types not inferred from `functions`, e.g., when using `void` pointers.
-/// - `patterns` - a list of [`LibraryPattern`](crate::patterns::LibraryPattern) to include `[p1, ...]`,
-///
-/// Any of `consts`, `functions` or `patters` can be an empty list `[]` instead. Most types are
-/// inferred automatically based on the used functions.
+/// You must also annotate the function with [`#[ffi_function]`](crate::ffi_function), and preferably with `#[no_mangle]` and `extern "C"`.
 ///
 /// # Example
 ///
 /// ```rust
-/// use interoptopus::{ffi_function, ffi_constant};
-///
-/// #[ffi_constant]
-/// const MY_CONSTANT: u8 = 1;
+/// use interoptopus::{ffi_function, Inventory, InventoryBuilder, function};
 ///
 /// #[ffi_function]
 /// #[no_mangle]
-/// pub extern "C" fn f(_x: u8) {}
+/// pub extern "C" fn my_function() { }
 ///
-/// interoptopus::inventory!(
-///     my_inventory_function,
-///     [ MY_CONSTANT ],
-///     [ f ],
-///     [], []
-/// );
+/// pub fn inventory() -> Inventory {
+///     InventoryBuilder::new()
+///         .register(function!(my_function))
+///         .inventory()
+/// }
 /// ```
-///
-/// You can then use `my_inventory_function`, which will return a [`Library`], in a backend to
-/// produce bindings to your language.
-///
 #[macro_export]
-macro_rules! inventory {
-    (
-        $export_function:ident,
-        [
-        $(
-            $const:path
-        ),* $(,)?
-        ],
-        [
-        $(
-            $function:path
-        ),* $(,)?
-        ],
-        [
-        $(
-            $extra_type:ty
-        ),* $(,)?
-        ],
-        [
-        $(
-            $pattern:path
-        ),* $(,)?
-        ]
-    ) => {
-        #[doc(hidden)]
-        pub fn $export_function() -> $crate::Library {
-            use $crate::lang::rust::FunctionInfo;
-            use $crate::lang::rust::ConstantInfo;
+macro_rules! function {
+    ($x:ty) => {{
+        use $crate::lang::rust::FunctionInfo;
+        // use $x as fnc;
+        let info = <$x>::function_info();
+        $crate::Symbol::Function(info)
+    }};
+}
 
-            let mut constants: ::std::vec::Vec<$crate::lang::c::Constant> = ::std::vec::Vec::new();
-            $(
-                {
-                    use $const as user_constant;
-                    constants.push(user_constant::constant_info());
-                }
-            )*
+/// Register a constant with an [`InventoryBuilder`].
+///
+/// You must also annotate the constant with [`#[ffi_constant]`](crate::ffi_constant).
+///
+/// # Example
+///
+/// ```rust
+/// use interoptopus::{ffi_constant, Inventory, InventoryBuilder, constant};
+///
+/// #[ffi_constant]
+/// pub const MY_CONSTANT: u32 = 123;
+///
+/// pub fn inventory() -> Inventory {
+///     InventoryBuilder::new()
+///         .register(constant!(MY_CONSTANT))
+///         .inventory()
+/// }
+/// ```
+#[macro_export]
+macro_rules! constant {
+    ($x:path) => {{
+        use $crate::lang::rust::ConstantInfo;
+        use $x as constant;
+        let info = constant::constant_info();
+        $crate::Symbol::Constant(info)
+    }};
+}
 
-            let mut functions: ::std::vec::Vec<$crate::lang::c::Function> = ::std::vec::Vec::new();
-            $(
-                {
-                    use $function as user_function;
-                    functions.push(user_function::function_info());
-                }
-            )*
+/// Register an extra type with an [`InventoryBuilder`].
+///
+/// You must also annotate the type with [`#[ffi_type]`](crate::ffi_type) and `#[repr(C)]`.
+///
+/// Note, most types are registered automatically. You only need this to pass types not directly visible in
+/// your function signatures, e.g., when accepting multiple types via a void pointer.
+///
+/// # Example
+///
+/// ```rust
+/// use interoptopus::{ffi_type, Inventory, InventoryBuilder, extra_type};
+///
+/// #[ffi_type]
+/// #[repr(C)]
+/// pub struct S<T> {
+///     t: T
+/// };
+///
+/// pub fn inventory() -> Inventory {
+///     InventoryBuilder::new()
+///         .register(extra_type!(S<f32>))
+///         .inventory()
+/// }
+#[macro_export]
+macro_rules! extra_type {
+    ($x:ty) => {{
+        let info = <$x as $crate::lang::rust::CTypeInfo>::type_info();
+        $crate::Symbol::Type(info)
+    }};
+}
 
-            let mut extra_types: ::std::vec::Vec<$crate::lang::c::CType> = ::std::vec::Vec::new();
-            $(
-                {
-                    extra_types.push(< $extra_type as $crate::lang::rust::CTypeInfo >::type_info() );
-                }
-            )*
-
-
-            let mut patterns: ::std::vec::Vec<$crate::patterns::LibraryPattern> = ::std::vec::Vec::new();
-            $(
-                {
-                    let pattern: $crate::patterns::LibraryPattern = < $pattern as  $crate::patterns::LibraryPatternInfo>::pattern_info();
-
-                    match &pattern {
-                        $crate::patterns::LibraryPattern::Service(class) => {
-                            functions.push(class.destructor().clone());
-                            functions.extend(class.constructors().iter().cloned());
-                            functions.extend(class.methods().iter().cloned());
-                        }
-                    }
-
-                    patterns.push(pattern);
-                }
-            )*
-
-
-            $crate::Library::new(functions, constants, patterns, extra_types)
-        }
-    };
+/// Register a pattern with an [`InventoryBuilder`].
+///
+/// You only need to register [`LibraryPattern`](crate::patterns::LibraryPattern), as [`TypePattern`](crate::patterns::TypePattern) are detected automatically.
+///
+/// # Example
+///
+/// Note, as this example focuses on the `pattern` macro it omits the definition of `Error` and `MyFFIError`.
+/// Their implementation can be found in the [FFIError](crate::patterns::result::FFIError) example.
+///
+/// ```rust
+/// use interoptopus::{ffi_type, ffi_service, ffi_service_ctor, Inventory, InventoryBuilder, pattern};
+///
+/// # use std::fmt::{Display, Formatter};
+/// # use interoptopus::patterns::result::FFIError;
+/// #
+/// # #[derive(Debug)]
+/// # pub enum Error {
+/// #     Bad,
+/// # }
+/// #
+/// # impl Display for Error {
+/// #     fn fmt(&self, _: &mut Formatter<'_>) -> std::fmt::Result {
+/// #         Ok(())
+/// #     }
+/// # }
+/// #
+/// # impl std::error::Error for Error {}
+/// #
+/// # #[ffi_type(patterns(ffi_error))]
+/// # #[repr(C)]
+/// # pub enum MyFFIError {
+/// #     Ok = 0,
+/// #     NullPassed = 1,
+/// #     Panic = 2,
+/// #     OtherError = 3,
+/// # }
+/// #
+/// # impl FFIError for MyFFIError {
+/// #     const SUCCESS: Self = Self::Ok;
+/// #     const NULL: Self = Self::NullPassed;
+/// #     const PANIC: Self = Self::Panic;
+/// # }
+/// #
+/// # impl From<Error> for MyFFIError {
+/// #     fn from(x: Error) -> Self {
+/// #         match x {
+/// #             Error::Bad => Self::OtherError,
+/// #         }
+/// #     }
+/// # }
+/// #
+///
+/// #[ffi_type(opaque)]
+/// pub struct SimpleService {
+///     pub some_value: u32,
+/// }
+///
+/// #[ffi_service(error = "MyFFIError", prefix = "simple_service_")]
+/// impl SimpleService {
+///
+///     #[ffi_service_ctor]
+///     pub fn new_with(some_value: u32) -> Result<Self, Error> {
+///         Ok(Self { some_value })
+///     }
+/// }
+///
+/// pub fn inventory() -> Inventory {
+///     InventoryBuilder::new()
+///         .register(pattern!(SimpleService))
+///         .inventory()
+/// }
+#[macro_export]
+macro_rules! pattern {
+    ($x:path) => {{
+        let info: $crate::patterns::LibraryPattern = <$x as $crate::patterns::LibraryPatternInfo>::pattern_info();
+        $crate::Symbol::Pattern(info)
+    }};
 }
